from os import PathLike
from pathlib import Path
from typing import IO, Any, Dict, Optional, Tuple, Union

from .base import BaseEntity, BaseCollectionProxy, Field

UploadFile = Union[str, PathLike, IO[bytes]]
DownloadPath = Union[str, PathLike]


def _write_download_response(response, path: DownloadPath, *, chunk_size: int = 8192) -> Path:
    destination = Path(path)
    destination.parent.mkdir(parents=True, exist_ok=True)

    try:
        with destination.open("wb") as handle:
            iter_content = getattr(response, "iter_content", None)

            if callable(iter_content):
                for chunk in iter_content(chunk_size=chunk_size):
                    if chunk:
                        handle.write(chunk)
            else:
                content = getattr(response, "content", b"")
                if content:
                    handle.write(content)
    finally:
        close = getattr(response, "close", None)
        if callable(close):
            close()

    return destination


class Artifact(BaseEntity):
    """
    Schema-backed entity for /artifacts.

    Artifacts are addressed by UUIDs instead of URNs and are bound to a parent
    entity via `parent_urn`.
    """

    ENDPOINT = "artifacts"
    IDENTIFIER_FIELD = "id"
    IMMUTABLE_FIELDS = {
        "id",
        "parent_urn",
        "type",
        "creator",
        "created_at",
        "updated_at",
    }

    id: str = Field("id", read_only=True)
    parent_urn: str = Field("parent_urn", read_only=True)
    title: str = Field("title", default="")
    description: Optional[str] = Field("description")
    type: str = Field("type", default="artifact", read_only=True)
    creator: Optional[str] = Field("creator", read_only=True)
    created_at: Optional[str] = Field("created_at", read_only=True)
    updated_at: Optional[str] = Field("updated_at", read_only=True)
    file_url: Optional[str] = Field("file_url")
    file_s3_url: Optional[str] = Field("file_s3_url")
    file_type: Optional[str] = Field("file_type")
    file_size: Optional[int] = Field("file_size")
    language: Optional[str] = Field("language")

    def save(self, *, only_dirty: bool = False) -> None:
        """
        Persist local changes.

        The server-side update schema requires `file_type`, so we include it
        whenever we are sending a partial artifact update.
        """
        if only_dirty and self._dirty_fields:
            body = {
                key: self.data[key]
                for key in self._dirty_fields
                if key in self.data and key not in self.IMMUTABLE_FIELDS
            }

            if "file_type" not in body and "file_type" in self.data:
                body["file_type"] = self.data["file_type"]

            if not body:
                return

            full = self.normalize_identifier(self.identifier)
            resp = self.client.patch(f"{self.ENDPOINT}/{full}", json=body)
            payload = resp.json()
            self.data = self._extract_result(payload)
            self._dirty_fields.clear()
            return

        super().save(only_dirty=only_dirty)

    def download(self, *, stream: bool = False, **kwargs):
        """Download the file associated with this artifact."""
        full = self.normalize_identifier(self.identifier)
        request_kwargs = dict(kwargs)
        if stream:
            request_kwargs["stream"] = True
        return self.client.request(
            "GET",
            f"{self.ENDPOINT}/{full}/download",
            **request_kwargs,
        )

    def download_to(
        self,
        path: DownloadPath,
        *,
        chunk_size: int = 8192,
        **kwargs,
    ) -> Path:
        """Download the file associated with this artifact to a local path."""
        response = self.download(stream=True, **kwargs)
        return _write_download_response(response, path, chunk_size=chunk_size)


class ArtifactsProxy(BaseCollectionProxy):
    ENTITY_CLS = Artifact
    ENDPOINT = "artifacts"

    def download(self, identifier: str, *, stream: bool = False, **kwargs):
        """Download the file associated with an artifact."""
        full = self.ENTITY_CLS.normalize_identifier(identifier)
        request_kwargs = dict(kwargs)
        if stream:
            request_kwargs["stream"] = True
        return self.client.request(
            "GET",
            f"{self.ENDPOINT}/{full}/download",
            **request_kwargs,
        )

    def download_to(
        self,
        identifier: str,
        path: DownloadPath,
        *,
        chunk_size: int = 8192,
        **kwargs,
    ) -> Path:
        """Download an artifact to a local path."""
        response = self.download(identifier, stream=True, **kwargs)
        return _write_download_response(response, path, chunk_size=chunk_size)

    def upload(
        self,
        file: UploadFile,
        *,
        parent_urn: str,
        title: Optional[str] = None,
        description: Optional[str] = None,
        language: Optional[str] = None,
    ) -> Artifact:
        """Upload a file and create an artifact in a single request."""
        data: Dict[str, Any] = {"parent_urn": parent_urn}
        if title is not None:
            data["title"] = title
        if description is not None:
            data["description"] = description
        if language is not None:
            data["language"] = language

        opened_file, file_payload = self._prepare_upload_file(file)
        try:
            resp = self.client.post(
                f"{self.ENDPOINT}/upload",
                data=data,
                files={"file": file_payload},
            )
        finally:
            if opened_file is not None:
                opened_file.close()

        payload = resp.json()
        result = self.ENTITY_CLS._extract_result(payload)
        return self.ENTITY_CLS(client=self.client, data=result)

    def _prepare_upload_file(
        self, file: UploadFile
    ) -> Tuple[Optional[IO[bytes]], Tuple[str, IO[bytes]]]:
        if isinstance(file, (str, PathLike)):
            path = Path(file)
            opened_file = path.open("rb")
            return opened_file, (path.name, opened_file)

        file_name = getattr(file, "name", "upload.bin")
        filename = Path(str(file_name)).name
        return None, (filename, file)


class ParentArtifactsProxy(ArtifactsProxy):
    def __init__(self, client, parent_urn: str) -> None:
        super().__init__(client)
        self.parent_urn = parent_urn

    def _fetch_urns(self, *, limit: int, offset: int = 0):
        resp = self.client.get(
            self.ENDPOINT,
            parent_urn=self.parent_urn,
            limit=limit,
            offset=offset,
        )
        payload = resp.json()
        return self._parse_list_result(payload)

    def _get_entity(self, urn: str, *, lazy: bool = False):
        entity = super()._get_entity(urn, lazy=lazy)
        if lazy:
            return entity
        if entity.parent_urn != self.parent_urn:
            raise KeyError(
                f"Artifact '{urn}' does not belong to parent '{self.parent_urn}'."
            )
        return entity

    def create(
        self,
        *,
        urn: Optional[str] = None,
        identifier: Optional[str] = None,
        **fields,
    ):
        payload = dict(fields)
        payload["parent_urn"] = self.parent_urn
        return super().create(
            urn=urn,
            identifier=identifier,
            **payload,
        )

    def upload(
        self,
        file: UploadFile,
        *,
        title: Optional[str] = None,
        description: Optional[str] = None,
        language: Optional[str] = None,
    ) -> Artifact:
        return super().upload(
            file,
            parent_urn=self.parent_urn,
            title=title,
            description=description,
            language=language,
        )



from typing import Any, Dict, List, Optional

from .base import BaseEntity, BaseCollectionProxy, Field


class Guide(BaseEntity):
    ENDPOINT = "guides"
    URN_PREFIX = "urn:guide:"
    IMMUTABLE_FIELDS = BaseEntity.IMMUTABLE_FIELDS | {"type"}

    id: str = Field("id", read_only=True)
    title: str = Field("title", default="")
    description: Optional[str] = Field("description")
    status: str = Field("status", default="active")
    type: str = Field("type", default="guide", read_only=True)
    tags: List[str] = Field("tags", default_factory=list)
    url: Optional[str] = Field("url")
    license: Optional[str] = Field("license")
    language: Optional[str] = Field("language")

    region: Optional[str] = Field("region")
    organization_urn: Optional[str] = Field("organization_urn")
    content: str = Field("content", default="")
    topic: Optional[str] = Field("topic")
    audience: Optional[str] = Field("audience")
    short_title: Optional[str] = Field("short_title")
    issuing_authority: Optional[str] = Field("issuing_authority")
    responsible_ministry: Optional[str] = Field("responsible_ministry")
    document_type: Optional[str] = Field("document_type")
    legal_status: Optional[str] = Field("legal_status")
    target_audiences: List[str] = Field("target_audiences", default_factory=list)
    graphical_model: Optional[str] = Field("graphical_model")
    evidence_basis: Optional[str] = Field("evidence_basis")
    notes: Optional[str] = Field("notes")
    review_status: Optional[str] = Field("review_status")
    verifier_user_id: Optional[str] = Field("verifier_user_id")
    visibility: Optional[str] = Field("visibility")
    applicability_status: Optional[str] = Field("applicability_status")
    applicability_start_date: Optional[str] = Field("applicability_start_date")
    applicability_end_date: Optional[str] = Field("applicability_end_date")
    publication_date: Optional[str] = Field("publication_date")
    publication_year: Optional[int] = Field("publication_year")
    revision: Optional[Dict[str, Any]] = Field("revision")
    identifiers: List[Dict[str, Any]] = Field("identifiers", default_factory=list)

    # Embedded relationship payloads, when present in API responses.
    artifact_records: List[Dict[str, Any]] = Field("artifacts", default_factory=list)
    guideline_ids: List[str] = Field("guidelines", default_factory=list)

    creator: Optional[str] = Field("creator", read_only=True)
    created_at: Optional[str] = Field("created_at", read_only=True)
    updated_at: Optional[str] = Field("updated_at", read_only=True)

    @property
    def guidelines(self):
        proxy = getattr(self, "_guidelines_proxy", None)
        if proxy is None:
            proxy = GuideGuidelinesProxy(self.client, guide_urn=self.urn)
            setattr(self, "_guidelines_proxy", proxy)
        return proxy

    @property
    def page(self):
        proxy = getattr(self, "_page_proxy", None)
        if proxy is None:
            proxy = GuidePageProxy(self.guidelines)
            setattr(self, "_page_proxy", proxy)
        return proxy


class Guideline(BaseEntity):
    ENDPOINT = "guidelines"
    IDENTIFIER_FIELD = "id"
    IMMUTABLE_FIELDS = {
        "id",
        "guide_urn",
        "guide_region",
        "creator",
        "created_at",
        "updated_at",
    }

    id: str = Field("id", read_only=True)
    guide_urn: str = Field("guide_urn", read_only=True)
    guide_region: Optional[str] = Field("guide_region", read_only=True)
    title: Optional[str] = Field("title")
    rule_text: str = Field("rule_text", default="")
    sequence_no: Optional[int] = Field("sequence_no")
    page_no: Optional[int] = Field("page_no")
    action_type: Optional[str] = Field("action_type")
    target_populations: List[Any] = Field("target_populations", default_factory=list)
    frequency: Optional[Any] = Field("frequency")
    quantity: Optional[Dict[str, Any]] = Field("quantity")
    food_groups: List[Any] = Field("food_groups", default_factory=list)
    source_refs: List[Dict[str, Any]] = Field("source_refs", default_factory=list)
    notes: Optional[str] = Field("notes")
    status: str = Field("status", default="active")
    review_status: Optional[str] = Field("review_status")
    verifier_user_id: Optional[str] = Field("verifier_user_id")
    visibility: Optional[str] = Field("visibility")
    applicability_status: Optional[str] = Field("applicability_status")
    applicability_start_date: Optional[str] = Field("applicability_start_date")
    applicability_end_date: Optional[str] = Field("applicability_end_date")
    creator: Optional[str] = Field("creator", read_only=True)
    created_at: Optional[str] = Field("created_at", read_only=True)
    updated_at: Optional[str] = Field("updated_at", read_only=True)


class GuidesProxy(BaseCollectionProxy):
    ENTITY_CLS = Guide
    ENDPOINT = "guides"


class GuidelinesProxy(BaseCollectionProxy):
    ENTITY_CLS = Guideline
    ENDPOINT = "guidelines"


class GuideGuidelinesProxy(GuidelinesProxy):
    def __init__(self, client, guide_urn: str) -> None:
        super().__init__(client)
        self.guide_urn = guide_urn

    @property
    def _by_guide_endpoint(self) -> str:
        return f"{self.ENDPOINT}/by-guide/{self.guide_urn}"

    def _fetch_urns(self, *, limit: int, offset: int = 0) -> List[str]:
        resp = self.client.get(
            self._by_guide_endpoint,
            limit=limit,
            offset=offset,
        )
        payload = resp.json()
        return self._parse_list_result(payload)

    def _get_entity(self, urn: str, *, lazy: bool = False) -> BaseEntity:
        entity = super()._get_entity(urn, lazy=lazy)
        if lazy:
            return entity
        if entity.guide_urn != self.guide_urn:
            raise KeyError(
                f"Guideline '{urn}' does not belong to guide '{self.guide_urn}'."
            )
        return entity

    def create(
        self,
        *,
        urn: Optional[str] = None,
        identifier: Optional[str] = None,
        **fields: Any,
    ) -> BaseEntity:
        payload = dict(fields)
        payload["guide_urn"] = self.guide_urn
        return super().create(
            urn=urn,
            identifier=identifier,
            **payload,
        )

    def _parse_search_results(self, payload: Any) -> List[Guideline]:
        result = BaseEntity._extract_result(payload)
        items = result.get("results", []) if isinstance(result, dict) else result

        if not isinstance(items, list):
            raise ValueError(f"Unexpected search response format: {items!r}")

        guidelines = []
        for item in items:
            if isinstance(item, dict) and "id" in item:
                guidelines.append(self.ENTITY_CLS(client=self.client, data=item))
            elif isinstance(item, str):
                guidelines.append(self._get_entity(item))
            else:
                raise ValueError(f"Unexpected guideline search item: {item!r}")

        return guidelines

    def search(
        self,
        q: str,
        fl: Optional[List[str]] = None,
        limit: int = 10,
        offset: int = 0,
        fq: Optional[List[str]] = None,
        sort: Optional[str] = None,
        fields: Optional[List[str]] = None,
        facet_limit: int = 50,
        highlight: bool = False,
        highlight_fields: Optional[List[str]] = None,
        highlight_pre_tag: str = "<em>",
        highlight_post_tag: str = "</em>",
    ) -> List[Guideline]:
        payload = {
            "q": q,
            "limit": limit,
            "offset": offset,
        }
        if fl is not None:
            payload["fl"] = fl
        if fq is not None:
            payload["fq"] = fq
        if sort is not None:
            payload["sort"] = sort
        if fields is not None:
            payload["fields"] = fields
        if facet_limit != 50:
            payload["facet_limit"] = facet_limit
        if highlight:
            payload["highlight"] = highlight
            if highlight_fields is not None:
                payload["highlight_fields"] = highlight_fields
            payload["highlight_pre_tag"] = highlight_pre_tag
            payload["highlight_post_tag"] = highlight_post_tag

        resp = self.client.post(f"{self._by_guide_endpoint}/search", json=payload)
        return self._parse_search_results(resp.json())

    def by_page(self, page_no: int) -> List[Guideline]:
        if not isinstance(page_no, int):
            raise TypeError(f"Page number must be an int, got {type(page_no)!r}.")
        if page_no < 0:
            raise ValueError("Page number must be non-negative.")

        filters = [
            f'guide_urn:"{self.guide_urn}"',
            f"page_no:{page_no}",
        ]
        resp = self.client.post(
            f"{self.ENDPOINT}/search",
            json={
                "limit": 1000,
                "offset": 0,
                "fq": filters,
            },
        )
        return self._parse_search_results(resp.json())


class GuidePageProxy:
    def __init__(self, guidelines: GuideGuidelinesProxy) -> None:
        self.guidelines = guidelines

    def __getitem__(self, page_no: int) -> List[Guideline]:
        return self.guidelines.by_page(page_no)



import types
from pathlib import Path

from wisefood.entities.articles import Article
from wisefood.entities.artifacts import Artifact, ArtifactsProxy, ParentArtifactsProxy
from wisefood.entities.guides import Guide

from conftest import DummyClient, StubResponse


ARTIFACT_ID = "123e4567-e89b-12d3-a456-426614174000"
GUIDE_URN = "urn:guide:mediterranean_guide"


class StreamingResponse:
    def __init__(self, chunks):
        self._chunks = chunks
        self.closed = False

    def iter_content(self, chunk_size=8192):
        yield from self._chunks

    def close(self):
        self.closed = True


def test_artifact_get_uses_uuid_endpoint(dummy_client: DummyClient):
    dummy_client.queue_response(
        "get",
        f"artifacts/{ARTIFACT_ID}",
        StubResponse(
            200,
            {
                "result": {
                    "id": ARTIFACT_ID,
                    "parent_urn": "urn:article:one",
                    "title": "Supplementary PDF",
                    "description": "Attached document",
                    "type": "artifact",
                    "file_url": "https://files.example.com/a.pdf",
                    "file_type": "application/pdf",
                    "file_size": 128,
                }
            },
        ),
    )

    artifact = Artifact.get(dummy_client, ARTIFACT_ID)

    assert artifact.id == ARTIFACT_ID
    assert artifact.parent_urn == "urn:article:one"
    assert artifact.title == "Supplementary PDF"


def test_artifacts_proxy_string_lookup_supports_uuid(monkeypatch):
    proxy = ArtifactsProxy(client=None)  # type: ignore[arg-type]
    proxy._urns = [ARTIFACT_ID]

    def get_entity(self, identifier, lazy=False):
        return f"entity:{identifier}"

    monkeypatch.setattr(proxy, "_get_entity", types.MethodType(get_entity, proxy))

    assert proxy[ARTIFACT_ID] == f"entity:{ARTIFACT_ID}"
    assert proxy.slugs() == [ARTIFACT_ID]


def test_artifact_save_only_dirty_includes_file_type(dummy_client: DummyClient):
    artifact = Artifact(
        client=dummy_client,
        data={
            "id": ARTIFACT_ID,
            "parent_urn": "urn:article:one",
            "title": "Old title",
            "description": "Attached document",
            "file_url": "https://files.example.com/a.pdf",
            "file_type": "application/pdf",
            "file_size": 128,
        },
        sync=False,
    )

    artifact.title = "New title"

    dummy_client.queue_response(
        "patch",
        f"artifacts/{ARTIFACT_ID}",
        StubResponse(
            200,
            {
                "result": {
                    "id": ARTIFACT_ID,
                    "parent_urn": "urn:article:one",
                    "title": "New title",
                    "description": "Attached document",
                    "file_url": "https://files.example.com/a.pdf",
                    "file_type": "application/pdf",
                    "file_size": 128,
                }
            },
        ),
    )

    artifact.save(only_dirty=True)

    method, endpoint, json_body, _kwargs = dummy_client.calls[-1]
    assert method == "patch"
    assert endpoint == f"artifacts/{ARTIFACT_ID}"
    assert json_body == {
        "title": "New title",
        "file_type": "application/pdf",
    }


def test_artifact_download_uses_download_endpoint(dummy_client: DummyClient):
    artifact = Artifact(
        client=dummy_client,
        data={
            "id": ARTIFACT_ID,
            "parent_urn": "urn:article:one",
            "title": "Supplementary PDF",
        },
        sync=False,
    )

    dummy_client.queue_response(
        "get",
        f"artifacts/{ARTIFACT_ID}/download",
        StubResponse(200, {"result": "https://files.example.com/download"}),
    )

    resp = artifact.download()

    assert resp.json()["result"] == "https://files.example.com/download"
    assert ("get", f"artifacts/{ARTIFACT_ID}/download", {}) in dummy_client.calls


def test_artifact_download_to_writes_local_file(dummy_client: DummyClient, tmp_path: Path):
    artifact = Artifact(
        client=dummy_client,
        data={
            "id": ARTIFACT_ID,
            "parent_urn": "urn:article:one",
            "title": "Supplementary PDF",
        },
        sync=False,
    )
    response = StreamingResponse([b"abc", b"", b"def"])
    dummy_client.queue_response("get", f"artifacts/{ARTIFACT_ID}/download", response)

    destination = tmp_path / "downloads" / "artifact.pdf"
    written_path = artifact.download_to(destination)

    assert written_path == destination
    assert destination.read_bytes() == b"abcdef"
    assert response.closed is True
    assert ("get", f"artifacts/{ARTIFACT_ID}/download", {"stream": True}) in dummy_client.calls


def test_artifacts_proxy_download_to_writes_local_file(
    dummy_client: DummyClient, tmp_path: Path
):
    proxy = ArtifactsProxy(dummy_client)
    response = StreamingResponse([b"guide", b"-pdf"])
    dummy_client.queue_response("get", f"artifacts/{ARTIFACT_ID}/download", response)

    destination = tmp_path / "client-level.pdf"
    written_path = proxy.download_to(ARTIFACT_ID, destination)

    assert written_path == destination
    assert destination.read_bytes() == b"guide-pdf"
    assert response.closed is True
    assert ("get", f"artifacts/{ARTIFACT_ID}/download", {"stream": True}) in dummy_client.calls


def test_parent_artifacts_proxy_download_to_writes_local_file(
    dummy_client: DummyClient, tmp_path: Path
):
    guide = Guide(
        client=dummy_client,
        data={"urn": GUIDE_URN, "title": "Mediterranean Guide"},
        sync=False,
    )
    response = StreamingResponse([b"page-1"])
    dummy_client.queue_response("get", f"artifacts/{ARTIFACT_ID}/download", response)

    destination = tmp_path / "guide-bound.pdf"
    written_path = guide.artifacts.download_to(ARTIFACT_ID, destination)

    assert written_path == destination
    assert destination.read_bytes() == b"page-1"
    assert response.closed is True
    assert ("get", f"artifacts/{ARTIFACT_ID}/download", {"stream": True}) in dummy_client.calls


def test_parent_entity_exposes_bound_artifacts_proxy(dummy_client: DummyClient):
    article = Article(
        client=dummy_client,
        data={"urn": "urn:article:one", "title": "Example"},
        sync=False,
    )

    dummy_client.queue_response(
        "get",
        "artifacts",
        StubResponse(200, {"result": [{"id": ARTIFACT_ID}]}),
    )

    proxy = article.artifacts
    artifacts = proxy[:1]

    assert isinstance(proxy, ParentArtifactsProxy)
    assert isinstance(artifacts[0], Artifact)
    assert artifacts[0].id == ARTIFACT_ID

    method, endpoint, kwargs = dummy_client.calls[-1]
    assert method == "get"
    assert endpoint == "artifacts"
    assert kwargs == {
        "parent_urn": "urn:article:one",
        "limit": 1,
        "offset": 0,
    }


def test_parent_entity_direct_artifact_lookup_validates_parent(dummy_client: DummyClient):
    article = Article(
        client=dummy_client,
        data={"urn": "urn:article:one", "title": "Example"},
        sync=False,
    )

    dummy_client.queue_response(
        "get",
        f"artifacts/{ARTIFACT_ID}",
        StubResponse(
            200,
            {
                "result": {
                    "id": ARTIFACT_ID,
                    "parent_urn": "urn:article:one",
                    "title": "Supplementary PDF",
                    "description": "Attached document",
                    "type": "artifact",
                    "file_url": "https://files.example.com/a.pdf",
                    "file_type": "application/pdf",
                    "file_size": 128,
                }
            },
        ),
    )

    artifact = article.artifacts[ARTIFACT_ID]

    assert artifact.id == ARTIFACT_ID
    assert artifact.parent_urn == "urn:article:one"


def test_parent_entity_artifacts_create_injects_parent_urn(dummy_client: DummyClient):
    article = Article(
        client=dummy_client,
        data={"urn": "urn:article:one", "title": "Example"},
        sync=False,
    )

    dummy_client.queue_response(
        "post",
        "artifacts",
        StubResponse(
            200,
            {
                "result": {
                    "id": ARTIFACT_ID,
                    "parent_urn": "urn:article:one",
                    "title": "Supplementary PDF",
                    "description": "Attached document",
                    "type": "artifact",
                    "file_url": "https://files.example.com/a.pdf",
                    "file_type": "application/pdf",
                    "file_size": 128,
                }
            },
        ),
    )

    created = article.artifacts.create(
        title="Supplementary PDF",
        description="Attached document",
        file_url="https://files.example.com/a.pdf",
        file_type="application/pdf",
        file_size=128,
    )

    method, endpoint, body, _kwargs = dummy_client.calls[-1]
    assert method == "post"
    assert endpoint == "artifacts"
    assert body["parent_urn"] == "urn:article:one"
    assert created.parent_urn == "urn:article:one"


def test_artifacts_proxy_upload_posts_multipart_form_data(
    dummy_client: DummyClient, tmp_path: Path
):
    file_path = tmp_path / "artifact.pdf"
    file_path.write_bytes(b"%PDF-1.4")

    dummy_client.queue_response(
        "post",
        "artifacts/upload",
        StubResponse(
            200,
            {
                "result": {
                    "id": ARTIFACT_ID,
                    "parent_urn": "urn:article:one",
                    "title": "Supplementary PDF",
                    "description": "Attached document",
                    "type": "artifact",
                    "file_url": "https://files.example.com/a.pdf",
                    "file_type": "application/pdf",
                    "file_size": 128,
                }
            },
        ),
    )

    proxy = ArtifactsProxy(dummy_client)
    created = proxy.upload(
        file_path,
        parent_urn="urn:article:one",
        title="Supplementary PDF",
        description="Attached document",
        language="en",
    )

    method, endpoint, json_body, kwargs = dummy_client.calls[-1]
    assert method == "post"
    assert endpoint == "artifacts/upload"
    assert json_body is None
    assert kwargs["data"] == {
        "parent_urn": "urn:article:one",
        "title": "Supplementary PDF",
        "description": "Attached document",
        "language": "en",
    }
    assert kwargs["files"]["file"][0] == "artifact.pdf"
    assert created.id == ARTIFACT_ID


def test_parent_artifacts_proxy_upload_injects_parent_urn(
    dummy_client: DummyClient, tmp_path: Path
):
    file_path = tmp_path / "artifact.pdf"
    file_path.write_bytes(b"%PDF-1.4")

    dummy_client.queue_response(
        "post",
        "artifacts/upload",
        StubResponse(
            200,
            {
                "result": {
                    "id": ARTIFACT_ID,
                    "parent_urn": "urn:article:one",
                    "title": "Supplementary PDF",
                    "description": "Attached document",
                    "type": "artifact",
                    "file_url": "https://files.example.com/a.pdf",
                    "file_type": "application/pdf",
                    "file_size": 128,
                }
            },
        ),
    )

    article = Article(
        client=dummy_client,
        data={"urn": "urn:article:one", "title": "Example"},
        sync=False,
    )
    created = article.artifacts.upload(file_path, title="Supplementary PDF")

    method, endpoint, json_body, kwargs = dummy_client.calls[-1]
    assert method == "post"
    assert endpoint == "artifacts/upload"
    assert json_body is None
    assert kwargs["data"]["parent_urn"] == "urn:article:one"
    assert created.parent_urn == "urn:article:one"
